// ======================================================================
// Filename:    AnimatedToon.fx
// Date:        2/17/2007
// Author:      John Beck
// Description: A new version of the Toon shader for Bunny Soccer with
//				support for vertex transformations used by the 
//				XNA Animation Component Library
// Copyright:    2007 No Solution.  All Rights Reserved.
// ======================================================================

float4x4	MatrixPalette[56];		// An array of bone transformations for our vertices. The
									// bones are mapped to indexes in the palette.
float4x4	World;					// Our world matrix
float4x4	View;					// The View Matrix
float4x4	Projection;				// The Projection Matrix
texture		BasicTexture;			// Incoming texture

float3		CameraPosition;			// Camera Position
float4		DiffuseLightColor;		// Diffuse light color
float3		LightPosition;			// Incoming Light Position

//Layer One is our base color, serves as the "shadowed" area of the surface
float LayerOneSharp;		// This sharpens the edges of the layer. Range of 0 to 1.
float LayerOneRough;		// This sets the bleed of the layer.
float LayerOneContrib;		// Multiplier for the final layer contribution. Range of 0 to 1. 

//Layer two is used to provide a specular highlight, or additional coloration
float LayerTwoSharp;		
float LayerTwoRough;		
float LayerTwoContrib;		
float EdgeThickness;		// Determines how thick of a line to draw around the object edges.

//=========================================================================== 
// STRUCTS
//=========================================================================== 

//Stores sampler state info for the texture
sampler TextureSampler = sampler_state
{
    AddressU  = WRAP;
    AddressV  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
	Texture = (BasicTexture);
};

struct VS_INPUT
{
	float4	Position	: POSITION0;		// Position of the vertex in the model file
	float3	Normal		: NORMAL0;			// vertex normal
	float2	Texcoord	: TEXCOORD0;		// texture coordinate for the vertex in the model file
	half4	Indices		: BLENDINDICES0;	// Indices of the bones that affect this vertex
	float4	Weights		: BLENDWEIGHT0;		// The weights for each bone that affects this vertex
};

// The Output for our skinning method
struct VS_OUTPUT1
{
	float4 Position			: POSITION0;
	float4 Normal			: TEXCOORD1;
	float3 ViewDirection	: TEXCOORD2;
	float3 LightDirection	: TEXCOORD3;
	float2 Texcoord			: TEXCOORD0;
};

// Output for the outline Vertex Shader
struct VS_OUTPUT2
{
	float4 Position			: POSITION0;
	float4 Normal			: TEXCOORD1;
};

// This is the output from our skinning method
struct SKIN_OUTPUT
{
    float4 Position;
    float4 Normal;
};

//=========================================================================== 
// METHODS
//=========================================================================== 

// This method takes in a vertex and applies the bone transforms to it.
SKIN_OUTPUT Skin4( const VS_INPUT Input)
{
    SKIN_OUTPUT Output = (SKIN_OUTPUT)0;
    // Since the weights need to add up to one, store 1.0 - (sum of the weights)
    float lastWeight = 1.0;
    float weight = 0;
    // Apply the transforms for the first 3 weights
    for (int i = 0; i < 3; ++i)
    {
        weight = Input.Weights[i];
        lastWeight -= weight;
        Output.Position     += mul( Input.Position, MatrixPalette[Input.Indices[i]]) * weight;
        Output.Normal       += mul( Input.Normal, MatrixPalette[Input.Indices[i]]) * weight;
    }
    // Apply the transform for the last weight
    Output.Position     += mul( Input.Position, MatrixPalette[Input.Indices[3]])*lastWeight;
    Output.Normal       += mul( Input.Normal, MatrixPalette[Input.Indices[3]])*lastWeight;
    return Output;
};


VS_OUTPUT1 Transform(VS_INPUT Input)
{
	VS_OUTPUT1 Output = (VS_OUTPUT1)0;
	SKIN_OUTPUT Skin = Skin4(Input);
	float4x4 WorldViewProjection = mul(World, mul(View, Projection));
	
	Output.Position = mul(Skin.Position, WorldViewProjection);
	Output.Normal = mul(Skin.Normal, World);

	float3 ObjectPosition = mul(Input.Position, World);		
	Output.ViewDirection = CameraPosition - ObjectPosition;
	Output.LightDirection = LightPosition - ObjectPosition;
	Output.Texcoord = Input.Texcoord;
	
	return Output;
}

// The Vertex Shader for our outline.
// Basically, we scale the model according to our edge offset, 
// and use the difference to draw our cool outline.
VS_OUTPUT2 Outline(VS_INPUT Input)
{
	SKIN_OUTPUT Skin = Skin4(Input);
	float4x4 WorldViewProjection = mul(mul(World, View), Projection);
	
	VS_OUTPUT2 Output;
	Output.Normal = mul(Skin.Normal, World);
	Output.Position = mul(Skin.Position, WorldViewProjection)+(mul(EdgeThickness, mul(Input.Normal, WorldViewProjection)));
	
	return Output;
}

// Input for the Toon Pixel Shader
struct PS_INPUT
{
	float3 Normal			: TEXCOORD1;
	float3 ViewDirection	: TEXCOORD2;
	float3 LightDirection	: TEXCOORD3;
	float2 Texcoord			: TEXCOORD0;
};

// The Toon Pixel Shader
// Calculates our surface color
float4 Cel(PS_INPUT Input) : COLOR0
{
	//Normalize our Input vectors
	float3 Normal			= normalize(Input.Normal);
	float3 ViewDirection	= normalize(Input.ViewDirection);
	float3 NLightDirection	= normalize(Input.LightDirection);
	
	float oneW = 0.18f * ( 1.0f - LayerOneSharp );
	float twoW = 0.18f * ( 1.0f - LayerTwoSharp );
	
	float NDotL				= dot(Normal, NLightDirection);
	float3 Reflection		= normalize(2.0f * NDotL * Normal - NLightDirection);
	
	float4 layerOneColor	= smoothstep(0.72f-oneW, 0.72f+oneW, pow(saturate(dot(Reflection, ViewDirection)), LayerOneRough));
	float4 layerTwoColor	= smoothstep(0.72f-twoW, 0.72f+twoW, pow(saturate(dot(Reflection, ViewDirection)), LayerTwoRough));
	
	float4 baseColor		= tex2D(TextureSampler, Input.Texcoord) * DiffuseLightColor;
	
	float4 color			= (baseColor + LayerOneContrib*layerOneColor) + LayerTwoContrib*layerTwoColor;
		
	return color;
}

// The Outline Pixel Shader.
float4 Black() : COLOR
{
   return float4(0.0f, 0.0f, 0.0f, 1.0f);
}
technique Toon
{
	pass P0
	{
		VertexShader = compile vs_2_0 Transform();
		PixelShader = compile ps_2_0 Cel();
		CullMode = NONE;
	}
	
	pass P1
	{
		VertexShader = compile vs_2_0 Outline();
		PixelShader  = compile ps_2_0 Black();
		CullMode = CW;
	}
}