// ======================================================================
// Filename:    ToonShader.cs
// Date:        2/17/2007
// Author:      John Beck
// Description: A Toon shader for Bunny Soccer.
// Copyright:    2007 No Solution.  All Rights Reserved.
// ======================================================================

float4x4 World;				// World Matrix
float4x4 View;				// View Matrix
float4x4 Projection;		// Projection Matrix
float3 CameraPosition;		// Camera Position
float4 DiffuseLightColor;	// Diffuse light color
float3 LightPosition;		// Incoming Light Position

//Layer One is our base color, serves as the "shadowed" area of the surface
float LayerOneSharp;		// This sharpens the edges of the layer. Range of 0 to 1.
float LayerOneRough;		// This sets the bleed of the layer.
float LayerOneContrib;		// Multiplier for the final layer contribution. Range of 0 to 1. 

//Layer two is used to provide a specular highlight, or additional coloration
float LayerTwoSharp;		
float LayerTwoRough;		
float LayerTwoContrib;		
float EdgeThickness;		// Determines how thick of a line to draw around the object edges.

//Sampler for the texture color applied to our model
sampler TextureSampler = sampler_state
{
    AddressU  = WRAP;
    AddressV  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

// Input for the Vertex Shaders
struct VS_INPUT
{
	float4 Position		: POSITION0;
	float3 Normal		: NORMAL0;
	float2 Texcoord		: TEXCOORD0;

};

// Output for the Toon Vertex shader
struct VS_OUTPUT1
{
	float4 Position			: POSITION0;
	float3 Normal			: TEXCOORD1;
	float3 ViewDirection	: TEXCOORD2;
	float3 LightDirection	: TEXCOORD3;
	float2 Texcoord			: TEXCOORD0;
};

// Output for the outline Vertex Shader
struct VS_OUTPUT2
{
	float4 Position			: POSITION0;
	float4 Normal			: TEXCOORD1;
};

// The Toon Vertex Shader
// Here, we basically just transform the model, and find the light and view directions
VS_OUTPUT1 Transform(VS_INPUT Input)
{
	float4x4 WorldViewProjection = mul(mul(World, View), Projection);
	float3 ObjectPosition = mul(Input.Position, World);
	
	VS_OUTPUT1 Output;
	Output.Normal = mul(Input.Normal, World);
	Output.Position = mul(Input.Position, WorldViewProjection);
	
	Output.ViewDirection = CameraPosition - ObjectPosition;
	Output.LightDirection = LightPosition - ObjectPosition;
	Output.Texcoord = Input.Texcoord;
	
	return Output;
}

// The Vertex Shader for our outline.
// Basically, we scale the model according to our edge offset, 
// and use the difference to draw our cool outline.
VS_OUTPUT2 Outline(VS_INPUT Input)
{
	float4x4 WorldViewProjection = mul(mul(World, View), Projection);
	
	VS_OUTPUT2 Output;
	Output.Normal = mul(Input.Normal, World);
	Output.Position = mul(Input.Position, WorldViewProjection)+(mul(EdgeThickness, mul(Input.Normal, WorldViewProjection)));
	
	return Output;
}

// Input for the Toon Pixel Shader
struct PS_INPUT
{
	float3 Normal			: TEXCOORD1;
	float3 ViewDirection	: TEXCOORD2;
	float3 LightDirection	: TEXCOORD3;
	float2 Texcoord			: TEXCOORD0;
};

// The Toon Pixel Shader
// Calculates our surface color
float4 Cel(PS_INPUT input) : COLOR0
{
	//Normalize our input vectors
	float3 Normal			= normalize(input.Normal);
	float3 ViewDirection	= normalize(input.ViewDirection);
	float3 NLightDirection	= normalize(input.LightDirection);
	
	float oneW = 0.18f * ( 1.0f - LayerOneSharp );
	float twoW = 0.18f * ( 1.0f - LayerTwoSharp );
	
	float NDotL				= dot(Normal, NLightDirection);
	float3 Reflection		= normalize(2.0f * NDotL * Normal - NLightDirection);
	
	float4 layerOneColor	= smoothstep(0.72f-oneW, 0.72f+oneW, pow(saturate(dot(Reflection, ViewDirection)), LayerOneRough));
	float4 layerTwoColor	= smoothstep(0.72f-twoW, 0.72f+twoW, pow(saturate(dot(Reflection, ViewDirection)), LayerTwoRough));
	
	float4 baseColor		= tex2D(TextureSampler, input.Texcoord) * DiffuseLightColor;
	
	float4 color			= (baseColor + LayerOneContrib*layerOneColor) + LayerTwoContrib*layerTwoColor;
	
	return color;
}

// The Outline Pixel Shader.
float4 Black() : COLOR
{
   return float4(0.0f, 0.0f, 0.0f, 1.0f);
}

technique Toon
{
	//Pass 0: render our surface color
	pass P0
	{
		VertexShader = compile vs_2_0 Transform();
		PixelShader  = compile ps_2_0 Cel();
		CullMode = NONE;
	}
	
	//Pass 1: render our outline
	pass P1
	{
		VertexShader = compile vs_2_0 Outline();
		PixelShader  = compile ps_2_0 Black();
		CullMode = CW;
	}
	
}