
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Xclna.Xna.Animation;                 // Don't forget to include the using statement!

#endregion

//===============================================================================
// Before getting started, don't forget to do the following:
// 1. add the "using Xclna.Xna.Animation;" to your Using Statements
//
// 2. Go to Project > Properties > Content Pipeline Tab > Add. Navigate to your
//    Xclna.Xna.Animation.Content.dll location.
//
// 3. In your solution explorer, right click on References and select Add Reference.
//    Under the Browse tab, navigate to Xclna.Xna.Animationx86.dll (for PC deployment),
//    select it, and hit ok.
//
// Your project is now ready to use the Animation Component Library
//===============================================================================

namespace IKTestBed
{
    public class IkTestBed : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        
        Model test;
        Effect toon;
        Texture2D JerseyTexture, BunnyTexture;
        Camera camera;

        #region Animation
        //===============================================================================
        // The stuff we need for our different animations. A bunnyAnimator for a given model,
        // and AnimationControllers for the different animations. Be sure to reference the xml
        // file for the corresponding animations, their duration, and time increments
        // (tickspersecond).
        //
        // Remember: The xml file that defines your animation segments must be in the same
        // folder as your model, and must be named <mymodelfilename>animation.xml
        // Example:
        //     Model:                   test.fbx
        //     Animation Definitions:   testanimation.xml
        //===============================================================================
        ModelAnimator bunnyAnimator;
        AnimationController current;
        AnimationController idle;
        AnimationController run;
        AnimationController jump;
        #endregion


        public IkTestBed()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }

        //===============================================================================
        // This is simply utility method to set the pone poses. This will generally be
        // called every update cycle.
        //===============================================================================
        private void runController(ModelAnimator animator, AnimationController controller)
        {
            foreach (BonePose pose in animator.BonePoses)
            {
                pose.CurrentController = controller;
                pose.CurrentBlendController = null;
            }
        }

        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            //===============================================================================
            // Our camera is a game component, so we need to be sure to add it to the 
            // game component selection. This basicallys means that the camera's update
            // method will be called without our having to manually do so ourselves.
            // I'm fairly certain that the Animation Component Library will import our model
            // as a DrawableGameComponent, meaning its Update() and Draw() methods will
            // be called automatically. To turn off updates and renders, you can manually set 
            // myComponent.Enabled to false.
            //===============================================================================
            camera = new Camera(this);
            this.Components.Add(camera);
            
            base.Initialize();
        }

        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                //===============================================================================
                // Load your content. Remember that you need to go to your model's properties,
                // and change the Content Importer and Content Processor to the Animation Library
                // versions, if one is available.
                //===============================================================================
                test = content.Load<Model>("content\\models\\test");
                toon = content.Load<Effect>("content\\effects\\AnimatedToon");
                JerseyTexture = content.Load<Texture2D>("content\\textures\\bunny_team1");
                BunnyTexture = content.Load<Texture2D>("content\\textures\\bunny");

                //===============================================================================
                // Now, here, you want to iterate through the parts of your mesh, and essentially
                // set the Effect member to your effect by cloning it. This is important incase
                // your different mesh parts needs different attributes that will be passed
                // to the shader at any given time. So for anything that isn't likely to change, 
                // set it here. Save dynamic paramters for the Draw loop.
                //
                // It is important to note that according to the authors, the ACL requires that
                // you set the View and Projection parameters BEFORE you set the World parameter,
                // which we will do later in the Draw method. We're doing it here simply to ensure
                // the View and Projection are set before we do anything else.
                //===============================================================================
                foreach (ModelMesh mesh in test.Meshes)
                {
                    for (int i = 0; i < mesh.MeshParts.Count; i++)
                    {
                        ModelMeshPart part = mesh.MeshParts[i];

                        part.Effect = toon.Clone(graphics.GraphicsDevice);
                        part.Effect.Parameters["View"].SetValue(camera.View);
                        part.Effect.Parameters["Projection"].SetValue(camera.Projection);
                        part.Effect.Parameters["LayerOneSharp"].SetValue(0.6f);
                        part.Effect.Parameters["LayerOneRough"].SetValue(0.0f);
                        part.Effect.Parameters["LayerOneContrib"].SetValue(0.05f);
                        part.Effect.Parameters["LayerTwoSharp"].SetValue(0.85f);
                        part.Effect.Parameters["LayerTwoRough"].SetValue(10.0f);
                        part.Effect.Parameters["LayerTwoContrib"].SetValue(0.3f);
                        part.Effect.Parameters["EdgeThickness"].SetValue(0.1f);
                        part.Effect.Parameters["DiffuseLightColor"].SetValue(new Vector4(0.8f,0.8f,0.8f,1.0f));
                        part.Effect.Parameters["LightPosition"].SetValue(new Vector3(10,100,10));
                        
                    }
                }

                //===============================================================================
                // Here, I'm setting a custom texture for the different mesh parts. Our
                // Bunnies have two parts: The bunny's body (MeshParts[0]: head, arms, legs), 
                // and the jersey (MeshParts[1]), each of which have their own texture.
                //===============================================================================
                test.Meshes[0].MeshParts[0].Effect.Parameters["BasicTexture"].SetValue(BunnyTexture);
                test.Meshes[0].MeshParts[1].Effect.Parameters["BasicTexture"].SetValue(JerseyTexture);

                //===============================================================================
                // Instantiate our animation stuff. I'm not sure if it's even necessary to 
                // call runController() here, honestly. I've taken it out
                //===============================================================================
                bunnyAnimator = new ModelAnimator(this, test);
                idle = new AnimationController(this, bunnyAnimator.Animations["idle"]);
                run = new AnimationController(this, bunnyAnimator.Animations["run"]);
                jump = new AnimationController(this, bunnyAnimator.Animations["jump"]);
                current = jump;
                // runController(bunnyAnimator, current);
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }
        
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }
        
        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            runController(bunnyAnimator, current);

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            graphics.GraphicsDevice.RenderState.MultiSampleAntiAlias = true;
            graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality = 4;
            graphics.GraphicsDevice.PresentationParameters.MultiSampleType = MultiSampleType.FourSamples;


            //===============================================================================
            // Iterate through your model's bits and pieces, set the effects, and go about your
            // bidness. 
            //===============================================================================
            foreach (ModelMesh mesh in test.Meshes)
            {
                for (int i = 0; i < mesh.MeshParts.Count; i++)
                {
                    ModelMeshPart part = mesh.MeshParts[i];
                    part.Effect.Parameters["World"].SetValue(Matrix.CreateRotationY(MathHelper.ToRadians(180)));
                    part.Effect.Parameters["View"].SetValue(camera.View);
                    part.Effect.Parameters["Projection"].SetValue(camera.Projection);
                    part.Effect.Parameters["CameraPosition"].SetValue(camera.Position);

                    //===============================================================================
                    // The authors advise that anytime you change the parameters, you call
                    // InitializeEffectParams(). I've commented it out, and noticed no change,
                    // honestly... but better safe than sorry, I suppose. 
                    //===============================================================================
                    bunnyAnimator.InitializeEffectParams();
                }
            }
            

            base.Draw(gameTime);
        }
    }
}